package logic

import (
	"encoding/json"
	"fmt"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"strings"

	"context"

	"fusenapi/server/info/internal/svc"
	"fusenapi/server/info/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
	"gorm.io/gorm"
)

type InfoLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewInfoLogic(ctx context.Context, svcCtx *svc.ServiceContext) *InfoLogic {
	return &InfoLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *InfoLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

// 这个与表名强关联
var ModuleTable map[string]string = map[string]string{
	"userinfo": "fs_user_info",
	"material": "fs_user_material",
}

type InfoType struct {
	Id       int64  `json:"id"`
	Module   string `json:"module"`
	Metadata string `json:"metadata"`
	// CreateAt time.Time `json:"ctime"`
}

func (l *InfoLogic) Info(req *types.UserInfoRequest, userinfo *auth.UserInfo) (resp *basic.Response) {
	// 返回值必须调用Set重新返回, resp可以空指针调用 resp.SetStatus(basic.CodeOK, data)
	// userinfo 传入值时, 一定不为null

	var cond string
	switch userinfo.GetIdType() {
	case auth.IDTYPE_User:
		cond = fmt.Sprintf("user_id = %d", userinfo.UserId)
	case auth.IDTYPE_Guest:
		cond = fmt.Sprintf("guest_id = %d", userinfo.GuestId)
	default:
		cond = "user_id = 0 and guest_id = 0"
	}

	var metadict map[string]any = make(map[string]any)

	for _, module := range req.Module {
		mlist := strings.Split(module, ".")
		if len(mlist) == 0 {
			return resp.SetStatusWithMessage(basic.CodeApiErr, fmt.Sprintf("%s format error", module))
		}

		mtable := mlist[0]
		tname, ok := ModuleTable[mtable]
		if !ok {
			return resp.SetStatusWithMessage(basic.CodeApiErr, fmt.Sprintf("%s format error, table %s not found", module, tname))
		}

		sqlstr := fmt.Sprintf("select id, module, metadata from %s where %s ", tname, cond)
		var info InfoType
		err := l.svcCtx.MysqlConn.Raw(sqlstr).Scan(&info).Error
		if err == gorm.ErrRecordNotFound {
			continue
		}

		if err != nil {
			logx.Error(err)
			return resp.SetStatusWithMessage(basic.CodeApiErr, err.Error())
		}

		var metadata map[string]any = make(map[string]any)
		err = json.Unmarshal([]byte(info.Metadata), &metadata)
		if err != nil {
			logx.Error(err)
			return resp.SetStatusWithMessage(basic.CodeApiErr, err.Error())
		}
		metadict[info.Module] = metadata
	}

	return resp.SetStatus(basic.CodeOK, metadict)
}

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *InfoLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }
